import 'dart:async';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_player/beans/player_info.dart';
import 'package:fijkplayer/fijkplayer.dart';

class SingleAudioUtils {
  factory SingleAudioUtils() => _getInstance();

  static SingleAudioUtils get instance => _getInstance();
  static SingleAudioUtils? _instance;

  static SingleAudioUtils _getInstance() {
    return _instance ??= SingleAudioUtils._internal();
  }

  SingleAudioUtils._internal();

  final Map<int, void Function()> _callMap = {};
  final Map<String, FijkPlayer> playMap = {};

  int playIndex = 0;

  PlayerInfo? _playerInfo;

  final StreamController<double> _playProgressController = StreamController.broadcast();

  //播放进度更新
  Stream<double> get onPlayProgressUpdate => _playProgressController.stream;

  /// 设置播放数据源
  void setSource({required PlayerInfo playerInfo}) {
    _playerInfo = playerInfo;
    var urlList = playerInfo.urlList;
    for (var element in urlList) {
      var player = _instancePlayer(element);
      var playerCode = element.hashCode;
      _callMap[playerCode] = () {
        if (player.state == FijkState.initialized) {
          playMap["${playerInfo.playerId}$playerCode"] = player;
          player.removeListener(_callMap[playerCode]!);
        } else if (player.state == FijkState.error) {
          player.removeListener(_callMap[playerCode]!);
        }
      };
      player.addListener(_callMap[playerCode]!);
      player.onCurrentPosUpdate.listen((event) => _playProgressListen(player.value.duration, event));
    }
  }

  void _playProgressListen(Duration total, Duration duration) {
    var currentTime = duration.inMilliseconds.toDouble();
    var totalTime = total.inMilliseconds.toDouble();
    var progress = currentTime / totalTime;
    _playProgressController.add(progress);
  }

  FijkPlayer _instancePlayer(String url) {
    return FijkPlayer()..setDataSource(url);
  }

  FijkPlayer? _getPlayer() {
    var urlList = _playerInfo?.urlList ?? [];
    if (urlList.isEmptyList || playIndex >= urlList.length) {
      return null;
    }
    var playCode = urlList[playIndex].hashCode;
    return playMap["${_playerInfo?.playerId ?? ""}$playCode"];
  }

  /// 获取播放器
  FijkPlayer? get player => _getPlayer();

  /// 开始播放
  void start() {
    var _player = player;
    var total = _player?.value.duration.inMilliseconds ?? 0;
    var milliseconds = _player?.currentPos.inMilliseconds ?? 0;
    if (milliseconds == total) {
      _player?.seekTo(0);
    } else {
      var targetSec = milliseconds - 500;
      if (targetSec > 0) {
        _player?.seekTo(targetSec);
      }
    }
    _player?.start();
  }

  /// 暂停播放
  void pause() {
    player?.pause();
  }
}
